from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PlayoutShow")


@attr.s(auto_attribs=True)
class PlayoutShow:
    """
    Attributes:
        id (int):
        name (str): Name of this Show.
        default_playlist_id (Union[Unset, None, int]):
    """

    id: int
    name: str
    default_playlist_id: Union[Unset, None, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        default_playlist_id = self.default_playlist_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )
        if default_playlist_id is not UNSET:
            field_dict["defaultPlaylistId"] = default_playlist_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        default_playlist_id = d.pop("defaultPlaylistId", UNSET)

        playout_show = cls(
            id=id,
            name=name,
            default_playlist_id=default_playlist_id,
        )

        playout_show.additional_properties = d
        return playout_show

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
