import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeSlot")


@attr.s(auto_attribs=True)
class TimeSlot:
    """
    Attributes:
        end (datetime.datetime):
        id (int):
        note_id (int):
        show_id (int):
        start (datetime.datetime):
        memo (Union[Unset, str]): Memo for this timeslot.
        playlist_id (Union[Unset, None, int]): Playlist ID of this timeslot.
        repetition_of_id (Union[Unset, None, int]): This timeslot is a repetition of `Timeslot` ID.
        schedule_id (Union[Unset, int]): `Schedule` ID of this timeslot.
    """

    end: datetime.datetime
    id: int
    note_id: int
    show_id: int
    start: datetime.datetime
    memo: Union[Unset, str] = UNSET
    playlist_id: Union[Unset, None, int] = UNSET
    repetition_of_id: Union[Unset, None, int] = UNSET
    schedule_id: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        end = self.end.isoformat()

        id = self.id
        note_id = self.note_id
        show_id = self.show_id
        start = self.start.isoformat()

        memo = self.memo
        playlist_id = self.playlist_id
        repetition_of_id = self.repetition_of_id
        schedule_id = self.schedule_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "end": end,
                "id": id,
                "noteId": note_id,
                "showId": show_id,
                "start": start,
            }
        )
        if memo is not UNSET:
            field_dict["memo"] = memo
        if playlist_id is not UNSET:
            field_dict["playlistId"] = playlist_id
        if repetition_of_id is not UNSET:
            field_dict["repetitionOfId"] = repetition_of_id
        if schedule_id is not UNSET:
            field_dict["scheduleId"] = schedule_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        end = isoparse(d.pop("end"))

        id = d.pop("id")

        note_id = d.pop("noteId")

        show_id = d.pop("showId")

        start = isoparse(d.pop("start"))

        memo = d.pop("memo", UNSET)

        playlist_id = d.pop("playlistId", UNSET)

        repetition_of_id = d.pop("repetitionOfId", UNSET)

        schedule_id = d.pop("scheduleId", UNSET)

        time_slot = cls(
            end=end,
            id=id,
            note_id=note_id,
            show_id=show_id,
            start=start,
            memo=memo,
            playlist_id=playlist_id,
            repetition_of_id=repetition_of_id,
            schedule_id=schedule_id,
        )

        time_slot.additional_properties = d
        return time_slot

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
